@@ -81,7 +81,6 @@ if TYPE_CHECKING:
 
     from numpy.typing import DTypeLike, NDArray
     from zarr.core.buffer import Buffer, BufferPrototype
-    from zarr.core.common import BytesLike
     from zarr.core.indexing import BasicSelection
 
     from .tifffile import ByteOrder, OutputType
@@ -97,11 +96,11 @@ class ZarrStore(Store):
         chunkmode:
             Specifies how to chunk data.
         read_only:
-            Passed to zarr `Store` constructor.
+            Passed to :py:class:`zarr.abc.store.Store`.
 
     References:
         1. https://zarr.readthedocs.io/en/stable/api/zarr/abc/store/
-        3. https://zarr.readthedocs.io/en/stable/spec/v2.html
+        2. https://zarr.readthedocs.io/en/stable/spec/v2.html
         3. https://forum.image.sc/t/multiscale-arrays-v0-1/37930
 
     """
@@ -171,17 +170,6 @@ class ZarrStore(Store):
         """Remove key from store."""
         raise PermissionError('ZarrStore does not support deletes')
 
-    @property
-    def supports_partial_writes(self) -> bool:
-        """Store support partial writes."""
-        return False
-
-    async def set_partial_values(
-        self, key_start_values: Iterable[tuple[str, int, BytesLike]]
-    ) -> None:
-        """Store values at key, starting at byte range_start."""
-        raise PermissionError('ZarrStore does not support partial writes')
-
     @property
     def supports_listing(self) -> bool:
         """Store supports listing."""
@@ -282,7 +270,7 @@ class ZarrTiffStore(ZarrStore):
             Approximate number of bytes to read from file in one pass
             if `chunkmode=2`. The default is :py:attr:`_TIFF.BUFFERSIZE`.
         read_only:
-            Passed to zarr `Store` constructor.
+            Passed to :py:class:`zarr.abc.store.Store`.
         _openfiles:
             Internal API.
 
@@ -415,7 +403,7 @@ class ZarrTiffStore(ZarrStore):
                     {
                         'zarr_format': 2,
                         'shape': shape,
-                        'chunks': _chunks(chunks, shape),
+                        'chunks': _chunks(chunks, shape, keyframe.shaped),
                         'dtype': _dtype_str(dtype),
                         'compressor': None,
                         'fill_value': _json_value(fillvalue, dtype),
@@ -445,7 +433,7 @@ class ZarrTiffStore(ZarrStore):
                 {
                     'zarr_format': 2,
                     'shape': shape,
-                    'chunks': _chunks(chunks, shape),
+                    'chunks': _chunks(chunks, shape, keyframe.shaped),
                     'dtype': _dtype_str(dtype),
                     'compressor': None,
                     'fill_value': _json_value(fillvalue, dtype),
@@ -888,9 +876,7 @@ class ZarrTiffStore(ZarrStore):
             self._filecache.close(fh)
             if self._transform is not None:
                 chunk = self._transform(chunk)
-            return prototype.buffer(
-                chunk.reshape(-1).view('B')  # type: ignore[arg-type]
-            )
+            return prototype.buffer(chunk.reshape(-1).view('B'))
 
         assert offset is not None and bytecount is not None
         chunk_bytes = self._filecache.read(fh, offset, bytecount)
@@ -921,9 +907,7 @@ class ZarrTiffStore(ZarrStore):
             chunks = keyframe.chunks
         if chunk.size != product(chunks):
             raise RuntimeError(f'{chunk.size} != {product(chunks)}')
-        return prototype.buffer(
-            chunk.reshape(-1).view('B')  # type: ignore[arg-type]
-        )
+        return prototype.buffer(chunk.reshape(-1).view('B'))
 
     async def exists(self, key: str) -> bool:
         """Return if key exists in store."""
@@ -1116,7 +1100,7 @@ class ZarrFileSequenceStore(ZarrStore):
             If enabled, internal threading for the `imread` function
             should be disabled.
         read_only:
-            Passed to zarr `Store` constructor.
+            Passed to :py:class:`zarr.abc.store.Store`.
         imreadargs:
             Arguments passed to :py:attr:`FileSequence.imread`.
         **kwargs:
@@ -1259,9 +1243,7 @@ class ZarrFileSequenceStore(ZarrStore):
             )
         else:
             chunk = self._imread(filename, **self._kwargs)
-        return prototype.buffer(
-            chunk.reshape(-1).view('B')  # type: ignore[arg-type]
-        )
+        return prototype.buffer(chunk.reshape(-1).view('B'))
 
     def write_fsspec(
         self,
@@ -1484,6 +1466,7 @@ def zarr_selection(
         else:
             ndbuffer = None
         result = zarray.get_basic_selection(selection, out=ndbuffer)
+        del zarray
     finally:
         if close:
             store.close()
@@ -1577,7 +1560,10 @@ def _is_writable(keyframe: TiffPage) -> bool:
 
 
 def _chunks(
-    chunks: tuple[int, ...], shape: tuple[int, ...], /
+    chunks: tuple[int, ...],
+    shape: tuple[int, ...],
+    shaped: tuple[int, int, int, int, int],
+    /,
 ) -> tuple[int, ...]:
     """Return chunks with same length as shape."""
     ndim = len(shape)
@@ -1585,30 +1571,24 @@ def _chunks(
         return ()  # empty array
     if 0 in shape:
         return (1,) * ndim
-    newchunks = []
-    i = ndim - 1
-    j = len(chunks) - 1
-    while True:
-        if j < 0:
-            newchunks.append(1)
-            i -= 1
-        elif shape[i] > 1 and chunks[j] > 1:
-            newchunks.append(chunks[j])
-            i -= 1
-            j -= 1
-        elif shape[i] == chunks[j]:  # both 1
-            newchunks.append(1)
-            i -= 1
-            j -= 1
-        elif shape[i] == 1:
-            newchunks.append(1)
-            i -= 1
-        elif chunks[j] == 1:
-            newchunks.append(1)
-            j -= 1
-        else:
-            raise RuntimeError
-        if i < 0 or ndim == len(newchunks):
-            break
-    # assert ndim == len(newchunks)
-    return tuple(newchunks[::-1])
+    d = 0 if shaped[1] == 1 else 1
+    i = min(ndim, 3 + d)
+    n = len(chunks)
+    if (
+        n == 2 + d
+        and i != 2 + d
+        and shape[-1] == 1
+        and shape[-i:] == shaped[-i:]
+    ):
+        # planarconfig=contig with one sample
+        chunks = chunks + (1,)
+    if ndim < len(chunks):
+        # remove leading dimensions of size 1 from chunks
+        for i, size in enumerate(chunks):
+            if size > 1:
+                break
+        chunks = chunks[i:]
+        if ndim < len(chunks):
+            raise ValueError(f'{shape=!r} is shorter than {chunks=!r}')
+    # prepend size 1 dimensions to chunks to match length of shape
+    return tuple([1] * (ndim - len(chunks)) + list(chunks))
